var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _CameraUpdate_native, _CameraPosition_native, _a, _MapView_delegate, _MapView_indoorDelegate, _MapView_didInit, _MapView_init, _MapView_isReady, _IndoorLevel_native, _IndoorBuilding_native, _UISettings_native, _GoogleMap_native, _GoogleMap_mapStyle, _Marker_native, _Marker_color, _Marker_icon, _Circle_native, _Polygon_native, _Polyline_native, _GroundOverlay_native, _GroundOverlay_image, _Poi_name, _Poi_placeId, _Poi_coord, _TileOverlay_native, _b, _Tile_native, _Tile_NONE, _TileProvider_native, _UrlTileProvider_native, _Projection_native, _VisibleRegion_native;
import { Color, ImageSource, Utils, View } from '@nativescript/core';
import { isNullOrUndefined } from '@nativescript/core/utils/types';
import { bearingProperty, latProperty, lngProperty, MapViewBase, tiltProperty, zoomProperty } from './common';
import { deserialize, intoNativeCircleOptions, intoNativeGroundOverlayOptions, intoNativeMarkerOptions, intoNativePolygonOptions, intoNativePolylineOptions, serialize } from './utils';
export class CameraUpdate {
    constructor() {
        _CameraUpdate_native.set(this, void 0);
    }
    static fromNative(nativeUpdate) {
        if (nativeUpdate instanceof GMSCameraUpdate) {
            const update = new CameraUpdate();
            __classPrivateFieldSet(update, _CameraUpdate_native, nativeUpdate, "f");
            return update;
        }
        return null;
    }
    static fromCoordinate(coordinate, zoom) {
        if (typeof zoom === 'number') {
            return CameraUpdate.fromNative(GMSCameraUpdate.setTargetZoom(CLLocationCoordinate2DMake(coordinate.lat, coordinate.lng), zoom));
        }
        else {
            return CameraUpdate.fromNative(GMSCameraUpdate.setTarget(CLLocationCoordinate2DMake(coordinate.lat, coordinate.lng)));
        }
    }
    static fromCameraPosition(position) {
        return CameraUpdate.fromNative(GMSCameraUpdate.setCamera(position.native));
    }
    static zoomIn() {
        return CameraUpdate.fromNative(GMSCameraUpdate.zoomIn());
    }
    static zoomOut() {
        return CameraUpdate.fromNative(GMSCameraUpdate.zoomOut());
    }
    static zoomTo(value) {
        return CameraUpdate.fromNative(GMSCameraUpdate.zoomTo(value));
    }
    static zoomBy(amount, point) {
        if (typeof (point === null || point === void 0 ? void 0 : point.x) === 'number' && typeof point.y === 'number') {
            return CameraUpdate.fromNative(GMSCameraUpdate.zoomByAtPoint(amount, CGPointMake(point.x, point.y)));
        }
        else {
            return CameraUpdate.fromNative(GMSCameraUpdate.zoomBy(amount));
        }
    }
    static scrollBy(x, y) {
        return CameraUpdate.fromNative(GMSCameraUpdate.scrollByXY(x, y));
    }
    get native() {
        return __classPrivateFieldGet(this, _CameraUpdate_native, "f");
    }
    get ios() {
        return this.native;
    }
}
_CameraUpdate_native = new WeakMap();
export class CameraPosition {
    constructor(target, zoom, bearing, tilt) {
        _CameraPosition_native.set(this, void 0);
        if (target && typeof zoom === 'number') {
            if (arguments.length === 2) {
                __classPrivateFieldSet(this, _CameraPosition_native, GMSCameraPosition.cameraWithLatitudeLongitudeZoom(target.lat, target.lng, zoom), "f");
            }
            else {
                if (typeof bearing === 'number' && typeof tilt === 'number') {
                    __classPrivateFieldSet(this, _CameraPosition_native, GMSCameraPosition.cameraWithLatitudeLongitudeZoomBearingViewingAngle(target.lat, target.lng, zoom, bearing, tilt), "f");
                }
                else {
                    const native = GMSCameraPosition.cameraWithLatitudeLongitudeZoom(target.lat, target.lng, zoom);
                    __classPrivateFieldSet(this, _CameraPosition_native, GMSCameraPosition.cameraWithTargetZoomBearingViewingAngle(native.target, native.zoom, bearing !== null && bearing !== void 0 ? bearing : native.bearing, tilt !== null && tilt !== void 0 ? tilt : native.viewingAngle), "f");
                }
            }
        }
    }
    static fromNative(nativePosition) {
        if (nativePosition instanceof GMSCameraPosition) {
            const position = new CameraPosition(null, null);
            __classPrivateFieldSet(position, _CameraPosition_native, nativePosition, "f");
            return position;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _CameraPosition_native, "f");
    }
    get ios() {
        return this.native;
    }
    get bearing() {
        return this.native.bearing;
    }
    set bearing(value) {
        __classPrivateFieldSet(this, _CameraPosition_native, GMSCameraPosition.cameraWithTargetZoomBearingViewingAngle(this.native.target, this.native.zoom, value, this.native.viewingAngle), "f");
    }
    get target() {
        return {
            lat: this.native.target.latitude,
            lng: this.native.target.longitude,
        };
    }
    set target(value) {
        __classPrivateFieldSet(this, _CameraPosition_native, GMSCameraPosition.cameraWithTargetZoomBearingViewingAngle(CLLocationCoordinate2DMake(value.lat, value.lng), this.native.zoom, this.native.bearing, this.native.viewingAngle), "f");
    }
    get tilt() {
        return this.native.viewingAngle;
    }
    set tilt(value) {
        __classPrivateFieldSet(this, _CameraPosition_native, GMSCameraPosition.cameraWithTargetZoomBearingViewingAngle(this.native.target, this.native.zoom, this.native.bearing, value), "f");
    }
    get zoom() {
        return this.native.zoom;
    }
    set zoom(value) {
        __classPrivateFieldSet(this, _CameraPosition_native, GMSCameraPosition.cameraWithTargetZoomBearingViewingAngle(this.native.target, value, this.native.bearing, this.native.viewingAngle), "f");
    }
    toJSON() {
        return {
            target: this.target,
            tilt: this.tilt,
            bearing: this.bearing,
            zoom: this.zoom,
        };
    }
}
_CameraPosition_native = new WeakMap();
var GMSMapViewDelegateImpl = /** @class */ (function (_super) {
    __extends(GMSMapViewDelegateImpl, _super);
    function GMSMapViewDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GMSMapViewDelegateImpl_1 = GMSMapViewDelegateImpl;
    GMSMapViewDelegateImpl.initWithOwner = function (owner) {
        var delegate = GMSMapViewDelegateImpl_1.new();
        delegate._owner = owner;
        delegate._pendingSnapshotResolveQueue = [];
        delegate._pendingSnapshotRejectQueue = [];
        return delegate;
    };
    GMSMapViewDelegateImpl.prototype.didTapMyLocationButtonForMapView = function (mapView) {
        var _a, _b, _c, _d, _e, _f;
        (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
            eventName: MapView.myLocationButtonTapEvent,
            object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
        });
        return true;
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidBeginDraggingMarker = function (mapView, marker) {
        var _a, _b, _c, _d, _e, _f;
        (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
            eventName: MapView.markerDragStartEvent,
            object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
            marker: Marker.fromNative(marker),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidChangeCameraPosition = function (mapView, position) {
        var _a, _b, _c, _d, _e, _f;
        (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
            eventName: MapView.cameraPositionEvent,
            state: 'moving',
            object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
            cameraPosition: CameraPosition.fromNative(position),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidCloseInfoWindowOfMarker = function (mapView, marker) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.infoWindowCloseEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            marker: Marker.fromNative(marker),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidDragMarker = function (mapView, marker) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.markerDraggingEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            marker: Marker.fromNative(marker),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidEndDraggingMarker = function (mapView, marker) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.markerDragEndEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            marker: Marker.fromNative(marker),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidFinishTileRendering = function (mapView) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.tileRenderingEndEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidLongPressAtCoordinate = function (mapView, coordinate) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.mapLongPressEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            coordinate: {
                lat: coordinate.latitude,
                lng: coordinate.longitude,
            },
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidLongPressInfoWindowOfMarker = function (mapView, marker) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.infoWindowLongPressEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            marker: Marker.fromNative(marker),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidStartTileRendering = function (mapView) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.tileRenderingStartEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidTapAtCoordinate = function (mapView, coordinate) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.mapTapEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            coordinate: {
                lat: coordinate.latitude,
                lng: coordinate.longitude,
            },
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidTapInfoWindowOfMarker = function (mapView, marker) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.infoWindowTapEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            marker: Marker.fromNative(marker),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidTapMarker = function (mapView, marker) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.markerTapEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            marker: Marker.fromNative(marker),
        });
        return false;
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidTapMyLocation = function (mapView, location) {
        var _a, _b, _c, _d;
        (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a).notify({
            eventName: MapView.markerTapEvent,
            object: (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : _d.call(_c),
            coordinate: {
                lat: location.latitude,
                lng: location.longitude,
            },
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidTapOverlay = function (mapView, overlay) {
        var _a, _b, _c, _d, _e, _f;
        if (overlay instanceof GMSGroundOverlay) {
            (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
                eventName: MapView.groundOverlayTapEvent,
                object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
                groundOverlay: GroundOverlay.fromNative(overlay),
            });
        }
    };
    GMSMapViewDelegateImpl.prototype.mapViewDidTapPOIWithPlaceIDNameLocation = function (mapView, placeID, name, location) {
        var _a, _b, _c, _d, _e, _f;
        (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
            eventName: MapView.poiTapEvent,
            object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
            poi: Poi.fromNative(placeID, name, location),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewIdleAtCameraPosition = function (mapView, position) {
        var _a, _b, _c, _d, _e, _f;
        (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
            eventName: MapView.cameraPositionEvent,
            state: 'idle',
            object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
            cameraPosition: CameraPosition.fromNative(position),
        });
    };
    GMSMapViewDelegateImpl.prototype.mapViewMarkerInfoContents = function (mapView, marker) {
        var _a, _b, _c, _d, _e, _f;
        var event = {
            eventName: MapView.markerInfoContentsEvent,
            object: (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a),
            marker: Marker.fromNative(marker),
            view: null,
        };
        (_f = (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : (_e = _d.call(_c)).notify) === null || _f === void 0 ? void 0 : _f.call(_e, event);
        if (event.view instanceof View) {
            return event.view.ios;
        }
        return null;
    };
    GMSMapViewDelegateImpl.prototype.mapViewMarkerInfoWindow = function (mapView, marker) {
        var _a, _b, _c, _d, _e, _f;
        var event = {
            eventName: MapView.markerInfoWindowEvent,
            object: (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : _b.call(_a),
            marker: Marker.fromNative(marker),
            view: null,
        };
        (_f = (_d = (_c = this._owner) === null || _c === void 0 ? void 0 : _c.get) === null || _d === void 0 ? void 0 : (_e = _d.call(_c)).notify) === null || _f === void 0 ? void 0 : _f.call(_e, event);
        if (event.view instanceof View) {
            return event.view.ios;
        }
        return null;
    };
    GMSMapViewDelegateImpl.prototype.mapViewSnapshotReady = function (mapView) {
        if (this._pendingSnapshotResolveQueue.length) {
            try {
                var format = UIGraphicsImageRendererFormat.defaultFormat();
                format.scale = UIScreen.mainScreen.scale;
                var renderer = UIGraphicsImageRenderer.alloc().initWithSizeFormat(mapView.frame.size, format);
                var image = renderer.imageWithActions(function (ctx) {
                    mapView.layer.renderInContext(ctx.CGContext);
                });
                var next = this._pendingSnapshotResolveQueue.shift();
                next(new ImageSource(image));
                this._pendingSnapshotRejectQueue.shift();
            }
            catch (e) {
                var next = this._pendingSnapshotResolveQueue.shift();
                next(e);
                this._pendingSnapshotResolveQueue.shift();
            }
        }
    };
    GMSMapViewDelegateImpl.prototype.mapViewWillMove = function (mapView, gesture) {
        var _a, _b, _c, _d, _e, _f;
        (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
            eventName: MapView.cameraPositionEvent,
            state: 'start',
            object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
            cameraPosition: CameraPosition.fromNative(mapView.camera),
            isGesture: gesture,
        });
    };
    var GMSMapViewDelegateImpl_1;
    GMSMapViewDelegateImpl = GMSMapViewDelegateImpl_1 = __decorate([
        ObjCClass(GMSMapViewDelegate)
    ], GMSMapViewDelegateImpl);
    return GMSMapViewDelegateImpl;
}(NSObject));
var GMSIndoorDisplayDelegateImpl = /** @class */ (function (_super) {
    __extends(GMSIndoorDisplayDelegateImpl, _super);
    function GMSIndoorDisplayDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GMSIndoorDisplayDelegateImpl_1 = GMSIndoorDisplayDelegateImpl;
    GMSIndoorDisplayDelegateImpl.initWithOwner = function (owner) {
        var delegate = GMSIndoorDisplayDelegateImpl_1.new();
        delegate._owner = owner;
        return delegate;
    };
    GMSIndoorDisplayDelegateImpl.prototype.didChangeActiveBuilding = function (building) {
        var _a, _b, _c, _d, _e, _f;
        (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
            eventName: MapView.activeBuildingEvent,
            object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
            building: IndoorBuilding.fromNative(building),
        });
    };
    GMSIndoorDisplayDelegateImpl.prototype.didChangeActiveLevel = function (level) {
        var _a, _b, _c, _d, _e, _f;
        (_d = (_b = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.get) === null || _b === void 0 ? void 0 : (_c = _b.call(_a)).notify) === null || _d === void 0 ? void 0 : _d.call(_c, {
            eventName: MapView.activeLevelEvent,
            object: (_f = (_e = this._owner) === null || _e === void 0 ? void 0 : _e.get) === null || _f === void 0 ? void 0 : _f.call(_e),
            level: IndoorLevel.fromNative(level),
        });
    };
    var GMSIndoorDisplayDelegateImpl_1;
    GMSIndoorDisplayDelegateImpl = GMSIndoorDisplayDelegateImpl_1 = __decorate([
        ObjCClass(GMSIndoorDisplayDelegate)
    ], GMSIndoorDisplayDelegateImpl);
    return GMSIndoorDisplayDelegateImpl;
}(NSObject));
var GMSSyncTileLayerImpl = /** @class */ (function (_super) {
    __extends(GMSSyncTileLayerImpl, _super);
    function GMSSyncTileLayerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GMSSyncTileLayerImpl.initWithCallback = function (callback) {
        var delegate = GMSSyncTileLayerImpl.new();
        delegate._callback = callback;
        return delegate;
    };
    GMSSyncTileLayerImpl.prototype.tileForXYZoom = function (x, y, zoom) {
        return this._callback(x, y, zoom).native;
    };
    return GMSSyncTileLayerImpl;
}(GMSSyncTileLayer));
export class MapView extends MapViewBase {
    constructor() {
        super(...arguments);
        _MapView_delegate.set(this, void 0);
        _MapView_indoorDelegate.set(this, void 0);
        _MapView_isReady.set(this, false);
    }
    createNativeView() {
        __classPrivateFieldGet(MapView, _a, "m", _MapView_init).call(MapView);
        const nativeView = GMSMapView.mapWithFrameCamera(CGRectZero, null);
        __classPrivateFieldSet(this, _MapView_delegate, GMSMapViewDelegateImpl.initWithOwner(new WeakRef(this)), "f");
        __classPrivateFieldSet(this, _MapView_indoorDelegate, GMSIndoorDisplayDelegateImpl.initWithOwner(new WeakRef(this)), "f");
        return nativeView;
    }
    initNativeView() {
        super.initNativeView();
        this.nativeView.delegate = __classPrivateFieldGet(this, _MapView_delegate, "f");
        this.nativeView.indoorDisplay.delegate = __classPrivateFieldGet(this, _MapView_indoorDelegate, "f");
    }
    onLoaded() {
        super.onLoaded();
        if (!__classPrivateFieldGet(this, _MapView_isReady, "f")) {
            this._updateCamera(this.nativeView, {
                lat: this.lat,
                lng: this.lng,
                bearing: this.bearing,
                tilt: this.tilt,
                zoom: this.zoom,
            });
            this.notify({
                eventName: 'ready',
                object: this,
                map: GoogleMap.fromNative(this.nativeView),
            });
            __classPrivateFieldSet(this, _MapView_isReady, true, "f");
        }
    }
    [(_MapView_delegate = new WeakMap(), _MapView_indoorDelegate = new WeakMap(), _MapView_isReady = new WeakMap(), _MapView_init = function _MapView_init() {
        if (__classPrivateFieldGet(this, _a, "f", _MapView_didInit)) {
            return;
        }
        const info = NSBundle.mainBundle.pathForResourceOfType('Info', '.plist');
        const dict = NSDictionary.dictionaryWithContentsOfFile(info);
        const APIKey = dict.objectForKey('TNSGoogleMapsAPIKey');
        if (APIKey) {
            GMSServices.provideAPIKey(APIKey);
        }
        else {
            console.error('Failed to load APIKey');
        }
        __classPrivateFieldSet(this, _a, true, "f", _MapView_didInit);
    }, latProperty.setNative)](value) {
        if (this.nativeView) {
            this._updateCamera(this.nativeView, {
                lat: value,
            });
        }
    }
    [lngProperty.setNative](value) {
        if (this.nativeView) {
            this._updateCamera(this.nativeView, {
                lng: value,
            });
        }
    }
    [zoomProperty.setNative](value) {
        if (this.nativeView) {
            this._updateCamera(this.nativeView, {
                zoom: value,
            });
        }
    }
    [tiltProperty.setNative](value) {
        if (this.nativeView) {
            this._updateCamera(this.nativeView, {
                tilt: value,
            });
        }
    }
    [bearingProperty.setNative](value) {
        if (this.nativeView) {
            this._updateCamera(this.nativeView, {
                bearing: value,
            });
        }
    }
    _updateCamera(map, owner) {
        const googleMap = GoogleMap.fromNative(map);
        if (googleMap) {
            const position = CameraPosition.fromNative(map.camera);
            let changed = false;
            if (!isNullOrUndefined(owner.lat)) {
                position.target = {
                    lat: typeof owner.lat === 'string' ? parseFloat(owner.lat) : owner.lat,
                    lng: position.target.lng,
                };
                changed = true;
            }
            if (!isNullOrUndefined(owner.lng)) {
                position.target = {
                    lat: position.target.lat,
                    lng: typeof owner.lng === 'string' ? parseFloat(owner.lng) : owner.lng,
                };
                changed = true;
            }
            if (!isNullOrUndefined(owner.zoom)) {
                position.zoom = typeof owner.zoom === 'string' ? parseFloat(owner.zoom) : owner.zoom;
                changed = true;
            }
            if (!isNullOrUndefined(owner.tilt)) {
                position.tilt = typeof owner.tilt === 'string' ? parseFloat(owner.tilt) : owner.tilt;
                changed = true;
            }
            if (!isNullOrUndefined(owner.bearing)) {
                position.bearing = typeof owner.bearing === 'string' ? parseFloat(owner.bearing) : owner.bearing;
                changed = true;
            }
            if (changed) {
                googleMap.cameraPosition = position;
            }
        }
    }
    onMeasure(widthMeasureSpec, heightMeasureSpec) {
        const nativeView = this.nativeView;
        if (nativeView) {
            const width = Utils.layout.getMeasureSpecSize(widthMeasureSpec);
            const height = Utils.layout.getMeasureSpecSize(heightMeasureSpec);
            this.setMeasuredDimension(width, height);
        }
    }
}
_a = MapView;
_MapView_didInit = { value: false };
export class IndoorLevel {
    constructor() {
        _IndoorLevel_native.set(this, void 0);
    }
    static fromNative(nativeIndoorLevel) {
        if (nativeIndoorLevel instanceof GMSIndoorLevel) {
            const indoorLevel = new IndoorLevel();
            __classPrivateFieldSet(indoorLevel, _IndoorLevel_native, nativeIndoorLevel, "f");
            return indoorLevel;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _IndoorLevel_native, "f");
    }
    get ios() {
        return this.native;
    }
    get name() {
        return this.native.name;
    }
    get shortName() {
        return this.native.shortName;
    }
}
_IndoorLevel_native = new WeakMap();
export class IndoorBuilding {
    constructor() {
        _IndoorBuilding_native.set(this, void 0);
    }
    static fromNative(nativeIndoorBuilding) {
        if (nativeIndoorBuilding instanceof GMSIndoorBuilding) {
            const indoorBuilding = new IndoorBuilding();
            __classPrivateFieldSet(indoorBuilding, _IndoorBuilding_native, nativeIndoorBuilding, "f");
            return indoorBuilding;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _IndoorBuilding_native, "f");
    }
    get ios() {
        return this.native;
    }
    get defaultLevelIndex() {
        return this.native.defaultLevelIndex;
    }
    get levels() {
        const result = [];
        const levels = this.native.levels;
        const count = levels.count;
        for (let i = 0; i < count; i++) {
            result.push(IndoorLevel.fromNative(levels.objectAtIndex(i)));
        }
        return result;
    }
    get underground() {
        return this.native.underground;
    }
}
_IndoorBuilding_native = new WeakMap();
export class UISettings {
    constructor() {
        _UISettings_native.set(this, void 0);
    }
    static fromNative(nativeUiSettings) {
        if (nativeUiSettings instanceof GMSUISettings) {
            const settings = new UISettings();
            __classPrivateFieldSet(settings, _UISettings_native, nativeUiSettings, "f");
            return settings;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _UISettings_native, "f");
    }
    get ios() {
        return this.native;
    }
    get zoomGesturesEnabled() {
        return this.native.zoomGestures;
    }
    set zoomGesturesEnabled(value) {
        this.native.zoomGestures = value;
    }
    get tiltGesturesEnabled() {
        return this.native.tiltGestures;
    }
    set tiltGesturesEnabled(value) {
        this.native.tiltGestures = value;
    }
    setAllGesturesEnabled(value) {
        this.native.setAllGesturesEnabled(value);
    }
    get compassEnabled() {
        return this.native.compassButton;
    }
    set compassEnabled(value) {
        this.native.compassButton = value;
    }
    get scrollGesturesEnabledDuringRotateOrZoom() {
        return this.native.allowScrollGesturesDuringRotateOrZoom;
    }
    set scrollGesturesEnabledDuringRotateOrZoom(value) {
        this.native.allowScrollGesturesDuringRotateOrZoom = value;
    }
    get rotateGesturesEnabled() {
        return this.native.rotateGestures;
    }
    set rotateGesturesEnabled(value) {
        this.native.rotateGestures = value;
    }
    get myLocationButtonEnabled() {
        return this.native.myLocationButton;
    }
    set myLocationButtonEnabled(value) {
        this.native.myLocationButton = value;
    }
    get indoorLevelPickerEnabled() {
        return this.native.indoorPicker;
    }
    set indoorLevelPickerEnabled(value) {
        this.native.indoorPicker = value;
    }
    get scrollGesturesEnabled() {
        return this.native.scrollGestures;
    }
    set scrollGesturesEnabled(value) {
        this.native.scrollGestures = value;
    }
}
_UISettings_native = new WeakMap();
export class GoogleMap {
    constructor() {
        _GoogleMap_native.set(this, void 0);
        _GoogleMap_mapStyle.set(this, void 0);
    }
    static fromNative(nativeMap) {
        if (nativeMap instanceof GMSMapView) {
            const map = new GoogleMap();
            __classPrivateFieldSet(map, _GoogleMap_native, nativeMap, "f");
            return map;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _GoogleMap_native, "f");
    }
    get ios() {
        return this.native;
    }
    get buildingsEnabled() {
        return this.native.buildingsEnabled;
    }
    set buildingsEnabled(value) {
        this.native.buildingsEnabled = value;
    }
    get cameraPosition() {
        return CameraPosition.fromNative(this.native.camera);
    }
    set cameraPosition(value) {
        this.native.camera = value.native;
    }
    get maxZoomLevel() {
        return this.native.maxZoom;
    }
    set maxZoomLevel(value) {
        this.native.setMinZoomMaxZoom(this.native.minZoom, value);
    }
    get minZoomLevel() {
        return this.native.minZoom;
    }
    set minZoomLevel(value) {
        this.native.setMinZoomMaxZoom(value, this.native.maxZoom);
    }
    get myLocationEnabled() {
        return this.native.myLocationEnabled;
    }
    set myLocationEnabled(value) {
        this.native.myLocationEnabled = value;
    }
    get trafficEnabled() {
        return this.native.trafficEnabled;
    }
    set trafficEnabled(value) {
        this.native.trafficEnabled = value;
    }
    get uiSettings() {
        return UISettings.fromNative(this.native.settings);
    }
    get mapStyle() {
        return __classPrivateFieldGet(this, _GoogleMap_mapStyle, "f");
    }
    set mapStyle(value) {
        try {
            const style = JSON.stringify(value);
            this.native.mapStyle = GMSMapStyle.styleWithJSONStringError(style);
        }
        catch (e) {
            console.error(e);
        }
    }
    clear() {
        this.native.clear();
    }
    addMarker(options) {
        const marker = Marker.fromNative(intoNativeMarkerOptions(options));
        if (options === null || options === void 0 ? void 0 : options.userData) {
            marker.userData = options.userData;
        }
        marker.native.map = this.native;
        return marker;
    }
    removeMarker(marker) {
        marker.native.map = null;
    }
    addCircle(options) {
        const circle = Circle.fromNative(intoNativeCircleOptions(options));
        circle.native.map = this.native;
        return circle;
    }
    addGroundOverlay(options) {
        const groundOverlay = GroundOverlay.fromNative(intoNativeGroundOverlayOptions(options));
        groundOverlay.native.map = this.native;
        return groundOverlay;
    }
    addPolygon(options) {
        const polygon = Polygon.fromNative(intoNativePolygonOptions(options));
        polygon.native.map = this.native;
        return polygon;
    }
    addPolyline(options) {
        const polyline = Polyline.fromNative(intoNativePolylineOptions(options));
        polyline.native.map = this.native;
        return polyline;
    }
    animateCamera(update) {
        this.native.animateWithCameraUpdate(update.native);
    }
    removeCircle(circle) {
        circle.native.map = null;
    }
    removeGroundOverlay(groundOverlay) {
        groundOverlay.native.map = null;
    }
    removePolygon(polygon) {
        polygon.native.map = null;
    }
    removePolyline(polyline) {
        polyline.native.map = null;
    }
    get projection() {
        return Projection.fromNative(this.native.projection);
    }
    snapshot() {
        return new Promise((resolve, reject) => {
            const delegate = this.native.delegate;
            delegate._pendingSnapshotRejectQueue.push(reject);
            delegate._pendingSnapshotResolveQueue.push(resolve);
            this.native.snapshotViewAfterScreenUpdates(true);
        });
    }
    addTileOverlay(options) {
        var _c, _d, _e;
        if ((_c = options === null || options === void 0 ? void 0 : options.tileProvider) === null || _c === void 0 ? void 0 : _c.native) {
            if (((_d = options === null || options === void 0 ? void 0 : options.tileProvider) === null || _d === void 0 ? void 0 : _d.native) instanceof GMSTileLayer) {
                const overlay = TileOverlay.fromNative((_e = options === null || options === void 0 ? void 0 : options.tileProvider) === null || _e === void 0 ? void 0 : _e.native);
                overlay.zIndex = options.zIndex;
                overlay.fadeIn = options.fadeIn;
                options.tileProvider.native.map = this.native;
                return overlay;
            }
        }
        return null;
    }
    removeTileOverlay(overlay) {
        overlay.native.map = null;
    }
}
_GoogleMap_native = new WeakMap(), _GoogleMap_mapStyle = new WeakMap();
class OverLayBase {
    get userData() {
        if (this.native.userData instanceof NSDictionary) {
            return deserialize(this.native.userData);
        }
        return {};
    }
    set userData(data) {
        if (data && typeof data === 'object') {
            this.native.userData = serialize(data);
        }
        else {
            this.native.userData = null;
        }
    }
}
export class Marker extends OverLayBase {
    constructor() {
        super(...arguments);
        _Marker_native.set(this, void 0);
        _Marker_color.set(this, new Color('red'));
        _Marker_icon.set(this, null);
    }
    static fromNative(nativeMarker) {
        if (nativeMarker instanceof GMSMarker) {
            const marker = new Marker();
            __classPrivateFieldSet(marker, _Marker_native, nativeMarker, "f");
            __classPrivateFieldSet(marker, _Marker_icon, new ImageSource(), "f");
            return marker;
        }
        return null;
    }
    get color() {
        return __classPrivateFieldGet(this, _Marker_color, "f");
    }
    set color(value) {
        if (value instanceof Color) {
            __classPrivateFieldSet(this, _Marker_color, value, "f");
            this.native.icon = GMSMarker.markerImageWithColor(value.ios);
        }
        else if (typeof value === 'string') {
            __classPrivateFieldSet(this, _Marker_color, new Color(value), "f");
            this.native.icon = GMSMarker.markerImageWithColor(__classPrivateFieldGet(this, _Marker_color, "f").ios);
        }
    }
    get native() {
        return __classPrivateFieldGet(this, _Marker_native, "f");
    }
    get ios() {
        return this.native;
    }
    get icon() {
        var _c;
        if (((_c = __classPrivateFieldGet(this, _Marker_icon, "f")) === null || _c === void 0 ? void 0 : _c.ios) === this.native.icon) {
            return __classPrivateFieldGet(this, _Marker_icon, "f");
        }
        else if (this.native.icon) {
            __classPrivateFieldGet(this, _Marker_icon, "f").setNativeSource(this.native.icon);
        }
        return __classPrivateFieldGet(this, _Marker_icon, "f");
    }
    set icon(value) {
        if (value instanceof UIImage) {
            this.native.icon = value;
            __classPrivateFieldGet(this, _Marker_icon, "f").setNativeSource(value);
        }
        else if (value instanceof ImageSource) {
            __classPrivateFieldSet(this, _Marker_icon, value, "f");
            this.native.icon = value.ios;
        }
    }
    get draggable() {
        return this.native.draggable;
    }
    set draggable(value) {
        this.native.draggable = value;
    }
    get position() {
        return {
            lat: this.native.position.latitude,
            lng: this.native.position.longitude,
        };
    }
    set position(latlng) {
        this.native.position = CLLocationCoordinate2DMake(latlng.lat, latlng.lng);
    }
    get title() {
        return this.native.title;
    }
    set title(value) {
        this.native.title = value;
    }
    get snippet() {
        return this.native.snippet;
    }
    set snippet(value) {
        this.native.snippet = value;
    }
    get rotation() {
        return this.native.rotation;
    }
    set rotation(value) {
        this.native.rotation = value;
    }
    get flat() {
        return this.native.flat;
    }
    set flat(value) {
        this.native.flat = value;
    }
    set zIndex(value) {
        this.native.zIndex = value;
    }
    get zIndex() {
        return this.native.zIndex;
    }
    hideInfoWindow() {
        const map = this.native.map;
        if ((map === null || map === void 0 ? void 0 : map.selectedMarker) === this.native) {
            map.selectedMarker = null;
        }
    }
    showInfoWindow() {
        const map = this.native.map;
        if (map) {
            map.selectedMarker = this.native;
        }
    }
}
_Marker_native = new WeakMap(), _Marker_color = new WeakMap(), _Marker_icon = new WeakMap();
export class Circle extends OverLayBase {
    constructor() {
        super(...arguments);
        _Circle_native.set(this, void 0);
    }
    static fromNative(nativeCircle) {
        if (nativeCircle instanceof GMSCircle) {
            const circle = new Circle();
            __classPrivateFieldSet(circle, _Circle_native, nativeCircle, "f");
            return circle;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _Circle_native, "f");
    }
    get ios() {
        return this.native;
    }
    get radius() {
        return this.native.radius;
    }
    set radius(value) {
        this.native.radius = value;
    }
    get strokeColor() {
        return Color.fromIosColor(this.native.strokeColor);
    }
    set strokeColor(value) {
        if (value instanceof Color) {
            this.native.strokeColor = value.ios;
        }
        else if (typeof value === 'string') {
            this.native.strokeColor = new Color(value).ios;
        }
    }
    get center() {
        const center = this.native.position;
        return {
            lat: center.latitude,
            lng: center.longitude,
        };
    }
    set center(value) {
        this.native.position = CLLocationCoordinate2DMake(value.lat, value.lng);
    }
    get fillColor() {
        return Color.fromIosColor(this.native.fillColor);
    }
    set fillColor(value) {
        if (value instanceof Color) {
            this.native.fillColor = value.ios;
        }
        else if (typeof value === 'string') {
            this.native.fillColor = new Color(value).ios;
        }
    }
    get tappable() {
        return this.native.tappable;
    }
    set tappable(value) {
        this.native.tappable = value;
    }
    get strokeWidth() {
        return this.native.strokeWidth;
    }
    set strokeWidth(value) {
        this.native.strokeWidth = value;
    }
    get zIndex() {
        return this.native.zIndex;
    }
    set zIndex(value) {
        this.native.zIndex = value;
    }
}
_Circle_native = new WeakMap();
export class Polygon extends OverLayBase {
    constructor() {
        super(...arguments);
        _Polygon_native.set(this, void 0);
    }
    static fromNative(nativePolygon) {
        if (nativePolygon instanceof GMSPolygon) {
            const polygon = new Polygon();
            __classPrivateFieldSet(polygon, _Polygon_native, nativePolygon, "f");
            return polygon;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _Polygon_native, "f");
    }
    get ios() {
        return this.native;
    }
    get points() {
        const count = this.native.path.count();
        const points = [];
        for (let i = 0; i < count; i++) {
            const point = this.native.path.coordinateAtIndex(i);
            points.push({
                lat: point.latitude,
                lng: point.longitude,
            });
        }
        return points;
    }
    set points(value) {
        const points = GMSMutablePath.path();
        if (Array.isArray(value)) {
            value.forEach((point) => {
                points.addCoordinate(CLLocationCoordinate2DMake(point.lat, point.lng));
            });
        }
        this.native.path = points;
    }
    get holes() {
        const count = this.native.holes.count;
        const holes = [];
        for (let i = 0; i < count; i++) {
            const hole = this.native.holes.objectAtIndex(i);
            const coord = hole.coordinateAtIndex(0);
            holes.push({
                lat: coord.latitude,
                lng: coord.longitude,
            });
        }
        return holes;
    }
    set holes(value) {
        const holes = [];
        if (Array.isArray(value)) {
            value.forEach((hole) => {
                const path = GMSMutablePath.path();
                path.addCoordinate(CLLocationCoordinate2DMake(hole.lat, hole.lng));
                holes.push(path);
            });
        }
        this.native.holes = holes;
    }
    get tappable() {
        return this.native.tappable;
    }
    set tappable(value) {
        this.native.tappable = value;
    }
    get strokeWidth() {
        return this.native.strokeWidth;
    }
    set strokeWidth(value) {
        this.native.strokeWidth = value;
    }
    get strokeColor() {
        return Color.fromIosColor(this.native.strokeColor);
    }
    set strokeColor(value) {
        if (value instanceof Color) {
            this.native.strokeColor = value.ios;
        }
        else if (typeof value === 'string') {
            this.native.strokeColor = new Color(value).ios;
        }
    }
    get zIndex() {
        return this.native.zIndex;
    }
    set zIndex(value) {
        this.native.zIndex = value;
    }
    get geodesic() {
        return this.native.geodesic;
    }
    set geodesic(value) {
        this.native.geodesic = value;
    }
    get fillColor() {
        return Color.fromIosColor(this.native.fillColor);
    }
    set fillColor(value) {
        if (value instanceof Color) {
            this.native.fillColor = value.ios;
        }
        else if (typeof value === 'string') {
            this.native.fillColor = new Color(value).ios;
        }
    }
}
_Polygon_native = new WeakMap();
export class Polyline extends OverLayBase {
    constructor() {
        super(...arguments);
        _Polyline_native.set(this, void 0);
    }
    static fromNative(nativePolyline) {
        if (nativePolyline instanceof GMSPolyline) {
            const polyline = new Polyline();
            __classPrivateFieldSet(polyline, _Polyline_native, nativePolyline, "f");
            return polyline;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _Polyline_native, "f");
    }
    get ios() {
        return this.native;
    }
    get width() {
        return this.native.strokeWidth;
    }
    set width(value) {
        this.native.strokeWidth = value;
    }
    get points() {
        const count = this.native.path.count();
        const points = [];
        for (let i = 0; i < count; i++) {
            const point = this.native.path.coordinateAtIndex(i);
            points.push({
                lat: point.latitude,
                lng: point.longitude,
            });
        }
        return points;
    }
    set points(value) {
        const points = GMSMutablePath.path();
        if (Array.isArray(value)) {
            value.forEach((point) => {
                points.addCoordinate(CLLocationCoordinate2DMake(point.lat, point.lng));
            });
        }
        this.native.path = points;
    }
    get tappable() {
        return this.native.tappable;
    }
    set tappable(value) {
        this.native.tappable = value;
    }
    get geodesic() {
        return this.native.geodesic;
    }
    set geodesic(value) {
        this.native.geodesic = value;
    }
    get zIndex() {
        return this.native.zIndex;
    }
    set zIndex(value) {
        this.native.zIndex = value;
    }
    get color() {
        return Color.fromIosColor(this.native.strokeColor);
    }
    set color(value) {
        if (value instanceof Color) {
            this.native.strokeColor = value.ios;
        }
        else if (typeof value === 'string') {
            this.native.strokeColor = new Color(value).ios;
        }
    }
}
_Polyline_native = new WeakMap();
export class GroundOverlay extends OverLayBase {
    constructor() {
        super();
        _GroundOverlay_native.set(this, void 0);
        _GroundOverlay_image.set(this, void 0);
        __classPrivateFieldSet(this, _GroundOverlay_image, new ImageSource(), "f");
    }
    static fromNative(nativeGroundOverlayOptions) {
        if (nativeGroundOverlayOptions instanceof GMSGroundOverlay) {
            const groundOverlayOptions = new GroundOverlay();
            __classPrivateFieldSet(groundOverlayOptions, _GroundOverlay_native, nativeGroundOverlayOptions, "f");
            return groundOverlayOptions;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _GroundOverlay_native, "f");
    }
    get ios() {
        return this.native;
    }
    get zIndex() {
        return this.native.zIndex;
    }
    set zIndex(value) {
        this.native.zIndex = value;
    }
    get position() {
        return {
            lat: this.native.position.latitude,
            lng: this.native.position.longitude,
        };
    }
    set position(value) {
        this.native.position = CLLocationCoordinate2DMake(value.lat, value.lng);
    }
    get anchorU() {
        return this.native.anchor.x;
    }
    set anchorU(value) {
        const anchor = this.native.anchor;
        this.native.anchor = CGPointMake(value, anchor.y);
    }
    get anchorV() {
        return this.native.anchor.y;
    }
    set anchorV(value) {
        const anchor = this.native.anchor;
        this.native.anchor = CGPointMake(anchor.x, value);
    }
    get bounds() {
        return {
            southwest: {
                lat: this.native.bounds.southWest.latitude,
                lng: this.native.bounds.southWest.longitude,
            },
            northeast: {
                lat: this.native.bounds.northEast.latitude,
                lng: this.native.bounds.northEast.longitude,
            },
        };
    }
    get tappable() {
        return this.native.tappable;
    }
    set tappable(value) {
        this.native.tappable = value;
    }
    get bearing() {
        return this.native.bearing;
    }
    set bearing(value) {
        this.native.bearing = value;
    }
    get image() {
        return __classPrivateFieldGet(this, _GroundOverlay_image, "f");
    }
    set image(value) {
        if (value instanceof UIImage) {
            this.native.icon = value;
            __classPrivateFieldGet(this, _GroundOverlay_image, "f").setNativeSource(value);
        }
        else if (value instanceof ImageSource) {
            __classPrivateFieldSet(this, _GroundOverlay_image, value, "f");
            this.native.icon = value.ios;
        }
    }
}
_GroundOverlay_native = new WeakMap(), _GroundOverlay_image = new WeakMap();
export class Poi {
    constructor() {
        _Poi_name.set(this, void 0);
        _Poi_placeId.set(this, void 0);
        _Poi_coord.set(this, void 0);
    }
    static fromNative(placeID, name, location) {
        if (arguments.length === 3) {
            const poi = new Poi();
            __classPrivateFieldSet(poi, _Poi_placeId, placeID, "f");
            __classPrivateFieldSet(poi, _Poi_name, name, "f");
            __classPrivateFieldSet(poi, _Poi_coord, location, "f");
        }
        return null;
    }
    get native() {
        return undefined;
    }
    get ios() {
        return this.native;
    }
    get coordinate() {
        return {
            lat: __classPrivateFieldGet(this, _Poi_coord, "f").latitude,
            lng: __classPrivateFieldGet(this, _Poi_coord, "f").longitude,
        };
    }
    get name() {
        return __classPrivateFieldGet(this, _Poi_name, "f");
    }
    get placeId() {
        return __classPrivateFieldGet(this, _Poi_placeId, "f");
    }
}
_Poi_name = new WeakMap(), _Poi_placeId = new WeakMap(), _Poi_coord = new WeakMap();
export class TileOverlay {
    constructor() {
        _TileOverlay_native.set(this, void 0);
    }
    static fromNative(nativeOverlay) {
        if (nativeOverlay instanceof GMSTileLayer) {
            const overlay = new TileOverlay();
            __classPrivateFieldSet(overlay, _TileOverlay_native, nativeOverlay, "f");
            return overlay;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _TileOverlay_native, "f");
    }
    get ios() {
        return this.native;
    }
    get fadeIn() {
        return this.native.fadeIn;
    }
    set fadeIn(value) {
        this.native.fadeIn = value;
    }
    get zIndex() {
        return this.native.zIndex;
    }
    set zIndex(value) {
        this.native.zIndex = value;
    }
}
_TileOverlay_native = new WeakMap();
export class Tile {
    constructor() {
        _Tile_native.set(this, void 0);
    }
    static get NONE() {
        if (!__classPrivateFieldGet(this, _b, "f", _Tile_NONE)) {
            __classPrivateFieldSet(this, _b, Tile.fromNative(kGMSTileLayerNoTile), "f", _Tile_NONE);
        }
        return __classPrivateFieldGet(this, _b, "f", _Tile_NONE);
    }
    static fromNative(nativeTile) {
        if (nativeTile instanceof UIImage) {
            const tileTile = new Tile();
            __classPrivateFieldSet(tileTile, _Tile_native, nativeTile, "f");
            return tileTile;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _Tile_native, "f");
    }
    get ios() {
        return this.native;
    }
}
_b = Tile, _Tile_native = new WeakMap();
_Tile_NONE = { value: void 0 };
export class TileProvider {
    constructor(callback) {
        _TileProvider_native.set(this, void 0);
        if (typeof callback === 'function') {
            __classPrivateFieldSet(this, _TileProvider_native, GMSSyncTileLayerImpl.initWithCallback(callback), "f");
        }
    }
    static fromNative(nativeTileProvider) {
        if (nativeTileProvider instanceof GMSSyncTileLayer) {
            const tileTileProvider = new TileProvider(null);
            __classPrivateFieldSet(tileTileProvider, _TileProvider_native, nativeTileProvider, "f");
            return tileTileProvider;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _TileProvider_native, "f");
    }
    get ios() {
        return this.native;
    }
}
_TileProvider_native = new WeakMap();
export class UrlTileProvider extends TileProvider {
    constructor(callback, size = 256) {
        super(null);
        _UrlTileProvider_native.set(this, void 0);
        const ref = new WeakRef(this);
        __classPrivateFieldSet(this, _UrlTileProvider_native, GMSURLTileLayer.tileLayerWithURLConstructor((x, y, zoom) => {
            return NSURL.URLWithString(callback(x, y, zoom));
        }), "f");
        __classPrivateFieldGet(this, _UrlTileProvider_native, "f").tileSize = size;
    }
    get native() {
        return __classPrivateFieldGet(this, _UrlTileProvider_native, "f");
    }
    get ios() {
        return this.native;
    }
}
_UrlTileProvider_native = new WeakMap();
export class Projection {
    constructor() {
        _Projection_native.set(this, void 0);
    }
    static fromNative(nativeProjection) {
        if (nativeProjection instanceof GMSProjection) {
            const projection = new Projection();
            __classPrivateFieldSet(projection, _Projection_native, nativeProjection, "f");
            return projection;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _Projection_native, "f");
    }
    get ios() {
        return this.native;
    }
    coordinateForPoint(point) {
        const location = this.native.coordinateForPoint(CGPointMake(point.x, point.y));
        return {
            lat: location.latitude,
            lng: location.longitude,
        };
    }
    visibleRegion() {
        return VisibleRegion.fromNative(this.native.visibleRegion());
    }
    pointForCoordinate(coordinate) {
        const point = this.native.pointForCoordinate(CLLocationCoordinate2DMake(coordinate.lat, coordinate.lng));
        return { x: point.x, y: point.y };
    }
}
_Projection_native = new WeakMap();
export class VisibleRegion {
    constructor() {
        _VisibleRegion_native.set(this, void 0);
    }
    static fromNative(nativeVisibleRegion) {
        if (nativeVisibleRegion instanceof GMSVisibleRegion) {
            const region = new VisibleRegion();
            __classPrivateFieldSet(region, _VisibleRegion_native, nativeVisibleRegion, "f");
            return region;
        }
        return null;
    }
    get native() {
        return __classPrivateFieldGet(this, _VisibleRegion_native, "f");
    }
    get ios() {
        return this.native;
    }
    get farLeft() {
        return {
            lat: this.native.farLeft.latitude,
            lng: this.native.farLeft.longitude,
        };
    }
    get farRight() {
        return {
            lat: this.native.farRight.latitude,
            lng: this.native.farRight.longitude,
        };
    }
    get nearLeft() {
        return {
            lat: this.native.nearLeft.latitude,
            lng: this.native.nearLeft.longitude,
        };
    }
    get nearRight() {
        return {
            lat: this.native.nearRight.latitude,
            lng: this.native.nearRight.longitude,
        };
    }
}
_VisibleRegion_native = new WeakMap();
export class PatternItem {
}
//# sourceMappingURL=index.ios.js.map