import { ImageSource, Utils } from '@nativescript/core';
import { intoNativeColor } from './common';
import { JointType } from '../common';
export function hueFromColor(color) {
    const colors = Array.create('float', 3);
    if (typeof color === 'number') {
        android.graphics.Color.colorToHSV(color, colors);
    }
    else {
        android.graphics.Color.colorToHSV(color.android, colors);
    }
    return colors[0];
}
export function toJointType(type) {
    switch (type) {
        case com.google.android.gms.maps.model.JointType.ROUND:
            return JointType.Round;
        case com.google.android.gms.maps.model.JointType.BEVEL:
            return JointType.Bevel;
        default:
            return JointType.Default;
    }
}
export function intoNativeJointType(type) {
    switch (type) {
        case JointType.Round:
            return com.google.android.gms.maps.model.JointType.ROUND;
        case JointType.Bevel:
            return com.google.android.gms.maps.model.JointType.BEVEL;
        default:
            return com.google.android.gms.maps.model.JointType.DEFAULT;
    }
}
export function intoNativeMarkerOptions(options) {
    var _a, _b;
    const opts = new com.google.android.gms.maps.model.MarkerOptions();
    if (typeof (options === null || options === void 0 ? void 0 : options.draggable) === 'boolean') {
        opts.draggable(options.draggable);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.anchorU) === 'number' || typeof (options === null || options === void 0 ? void 0 : options.anchorV) === 'number') {
        const anchorU = (_a = options === null || options === void 0 ? void 0 : options.anchorU) !== null && _a !== void 0 ? _a : opts.getAnchorU();
        const anchorV = (_b = options === null || options === void 0 ? void 0 : options.anchorV) !== null && _b !== void 0 ? _b : opts === null || opts === void 0 ? void 0 : opts.getAnchorV();
        opts.anchor(anchorU, anchorV);
    }
    if (options === null || options === void 0 ? void 0 : options.position) {
        opts.position(new com.google.android.gms.maps.model.LatLng(options.position.lat, options.position.lng));
    }
    if (options === null || options === void 0 ? void 0 : options.title) {
        opts.title(options.title);
    }
    if (options === null || options === void 0 ? void 0 : options.snippet) {
        opts.snippet(options.snippet);
    }
    if (options === null || options === void 0 ? void 0 : options.icon) {
        if ((options === null || options === void 0 ? void 0 : options.icon) instanceof android.graphics.Bitmap) {
            const desc = com.google.android.gms.maps.model.BitmapDescriptorFactory.fromBitmap(options.icon);
            opts.icon(desc);
        }
        else if ((options === null || options === void 0 ? void 0 : options.icon) instanceof ImageSource) {
            const desc = com.google.android.gms.maps.model.BitmapDescriptorFactory.fromBitmap(options.icon.android);
            opts.icon(desc);
        }
    }
    const color = intoNativeColor(options.color);
    if (color !== null) {
        opts.icon(com.google.android.gms.maps.model.BitmapDescriptorFactory.defaultMarker(hueFromColor(color)));
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.rotation) === 'number') {
        opts.rotation(options.rotation);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.flat) === 'boolean') {
        opts.flat(options.flat);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex(options.zIndex);
    }
    return opts;
}
export function intoNativeCircleOptions(options) {
    const opts = new com.google.android.gms.maps.model.CircleOptions();
    if (typeof (options === null || options === void 0 ? void 0 : options.radius) === 'number') {
        opts.radius(options.radius);
    }
    const strokeColor = intoNativeColor(options === null || options === void 0 ? void 0 : options.strokeColor);
    if (strokeColor !== null) {
        opts.strokeColor(strokeColor);
    }
    const fillColor = intoNativeColor(options === null || options === void 0 ? void 0 : options.fillColor);
    if (fillColor !== null) {
        opts.fillColor(fillColor);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.clickable(options.tappable);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.strokeWidth) === 'number') {
        opts.strokeWidth(Utils.layout.toDevicePixels(options.strokeWidth));
    }
    if (options === null || options === void 0 ? void 0 : options.center) {
        opts.center(new com.google.android.gms.maps.model.LatLng(options.center.lat, options.center.lng));
    }
    if (Array.isArray(options.strokePattern)) {
        const list = new java.util.ArrayList();
        options.strokePattern.forEach((item) => {
            list.add(item.native);
        });
        opts.strokePattern(list);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex(options.zIndex);
    }
    return opts;
}
export function intoNativePolygonOptions(options) {
    const opts = new com.google.android.gms.maps.model.PolygonOptions();
    if (Array.isArray(options === null || options === void 0 ? void 0 : options.points)) {
        options.points.forEach((point) => {
            opts.add(new com.google.android.gms.maps.model.LatLng(point.lat, point.lng));
        });
    }
    if (Array.isArray(options === null || options === void 0 ? void 0 : options.holes)) {
        const holes = new java.util.ArrayList();
        options.holes.forEach((hole) => {
            holes.add(new com.google.android.gms.maps.model.LatLng(hole.lat, hole.lng));
        });
        if (options.holes.length) {
            opts.addHole(holes);
        }
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.clickable(options.tappable);
    }
    const strokeColor = intoNativeColor(options === null || options === void 0 ? void 0 : options.strokeColor);
    if (strokeColor !== null) {
        opts.strokeColor(strokeColor);
    }
    const fillColor = intoNativeColor(options === null || options === void 0 ? void 0 : options.fillColor);
    if (fillColor !== null) {
        opts.fillColor(fillColor);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.strokeWidth) === 'number') {
        opts.strokeWidth(Utils.layout.toDevicePixels(options.strokeWidth));
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.geodesic) === 'boolean') {
        opts.geodesic(options.geodesic);
    }
    if (options === null || options === void 0 ? void 0 : options.strokeJointType) {
        opts.strokeJointType(intoNativeJointType(options.strokeJointType));
    }
    if (Array.isArray(options.strokePattern)) {
        const list = new java.util.ArrayList();
        options.strokePattern.forEach((item) => {
            list.add(item.native);
        });
        opts.strokePattern(list);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex(options.zIndex);
    }
    return opts;
}
export function intoNativePolylineOptions(options) {
    const opts = new com.google.android.gms.maps.model.PolylineOptions();
    if (typeof (options === null || options === void 0 ? void 0 : options.width) === 'number') {
        opts.width(Utils.layout.toDevicePixels(options.width));
    }
    if (Array.isArray(options === null || options === void 0 ? void 0 : options.points)) {
        options.points.forEach((point) => {
            opts.add(new com.google.android.gms.maps.model.LatLng(point.lat, point.lng));
        });
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.clickable(options.tappable);
    }
    const color = intoNativeColor(options === null || options === void 0 ? void 0 : options.color);
    if (color !== null) {
        opts.color(color);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.geodesic) === 'boolean') {
        opts.geodesic(options.geodesic);
    }
    if (options === null || options === void 0 ? void 0 : options.jointType) {
        opts.jointType(intoNativeJointType(options.jointType));
    }
    if (Array.isArray(options.pattern)) {
        const list = new java.util.ArrayList();
        options.pattern.forEach((item) => {
            list.add(item.native);
        });
        opts.pattern(list);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex(options.zIndex);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.startCap)) {
        opts.startCap(options.startCap.native);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.endCap)) {
        opts.endCap(options.endCap.native);
    }
    return opts;
}
export function intoNativeGroundOverlayOptions(options) {
    var _a, _b;
    const opts = new com.google.android.gms.maps.model.GroundOverlayOptions();
    if (typeof (options === null || options === void 0 ? void 0 : options.width) === 'number') {
        opts.position(opts.getLocation(), options.width);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.height) === 'number') {
        opts.position(opts.getLocation(), opts.getWidth(), options.height);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.transparency)) {
        opts.transparency(options.transparency);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.anchorU) === 'number' || typeof (options === null || options === void 0 ? void 0 : options.anchorV) === 'number') {
        opts.anchor((_a = options === null || options === void 0 ? void 0 : options.anchorU) !== null && _a !== void 0 ? _a : opts.getAnchorU(), (_b = options === null || options === void 0 ? void 0 : options.anchorV) !== null && _b !== void 0 ? _b : opts.getAnchorV());
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.clickable(options.tappable);
    }
    if (options === null || options === void 0 ? void 0 : options.position) {
        const coords = options.position;
        opts.position(new com.google.android.gms.maps.model.LatLng(coords.lat, coords.lng), opts.getWidth());
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.clickable(options.tappable);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.bearing) === 'number') {
        opts.bearing(options.bearing);
    }
    if ((options === null || options === void 0 ? void 0 : options.image) instanceof android.graphics.Bitmap) {
        opts.image(com.google.android.gms.maps.model.BitmapDescriptorFactory.fromBitmap(options === null || options === void 0 ? void 0 : options.image));
    }
    else if ((options === null || options === void 0 ? void 0 : options.image) instanceof ImageSource) {
        opts.image(com.google.android.gms.maps.model.BitmapDescriptorFactory.fromBitmap(options === null || options === void 0 ? void 0 : options.image.android));
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex(options.zIndex);
    }
    return opts;
}
export function intoNativeTileOverlayOptions(options) {
    const opts = new com.google.android.gms.maps.model.TileOverlayOptions();
    if (typeof (options === null || options === void 0 ? void 0 : options.fadeIn) === 'boolean') {
        opts.fadeIn(options.fadeIn);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.transparency) === 'number') {
        opts.transparency(options.transparency);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.visible) === 'number') {
        opts.visible(options.visible);
    }
    if (options === null || options === void 0 ? void 0 : options.tileProvider) {
        opts.tileProvider(options.tileProvider.native);
    }
    if (options === null || options === void 0 ? void 0 : options.zIndex) {
        opts.zIndex(options.zIndex);
    }
    return opts;
}
export function deserialize(data) {
    if (data === null || typeof data !== 'object') {
        return data;
    }
    let store;
    switch (data.getClass().getName()) {
        case 'java.lang.String': {
            return String(data);
        }
        case 'java.lang.Boolean': {
            return String(data) === 'true';
        }
        case 'java.lang.Float':
        case 'java.lang.Integer':
        case 'java.lang.Long':
        case 'java.lang.Double':
        case 'java.lang.Short': {
            return Number(data);
        }
        case 'org.json.JSONArray': {
            store = [];
            for (let j = 0; j < data.length(); j++) {
                store[j] = deserialize(data.get(j));
            }
            break;
        }
        case 'org.json.JSONObject': {
            store = {};
            let i = data.keys();
            while (i.hasNext()) {
                let key = i.next();
                store[key] = deserialize(data.get(key));
            }
            break;
        }
        case 'androidx.collection.SimpleArrayMap': {
            const count = data.size();
            for (let l = 0; l < count; l++) {
                const key = data.keyAt(l);
                store[key] = deserialize(data.get(key));
            }
            break;
        }
        case 'androidx.collection.ArrayMap':
        case 'android.os.Bundle':
        case 'java.util.HashMap':
        case 'java.util.Map': {
            store = {};
            const keys = data.keySet().toArray();
            for (let k = 0; k < keys.length; k++) {
                const key = keys[k];
                store[key] = deserialize(data.get(key));
            }
            break;
        }
        case 'java.util.Date': {
            return new Date(data.getTime());
        }
        default:
            if (typeof data === 'object' && data instanceof java.util.List) {
                const array = [];
                const size = data.size();
                for (let i = 0, n = size; i < n; i++) {
                    array[i] = deserialize(data.get(i));
                }
                store = array;
            }
            else {
                store = null;
            }
            break;
    }
    return store;
}
function numberHasDecimals(item) {
    return !(item % 1 === 0);
}
function numberIs64Bit(item) {
    return item < -Math.pow(2, 31) + 1 || item > Math.pow(2, 31) - 1;
}
export function serialize(data) {
    let store;
    switch (typeof data) {
        case 'string':
        case 'boolean': {
            if (typeof data === 'string') {
                return new java.lang.String(data);
            }
            return new java.lang.Boolean(data);
        }
        case 'number': {
            const hasDecimals = numberHasDecimals(data);
            if (numberIs64Bit(data)) {
                if (hasDecimals) {
                    return java.lang.Double.valueOf(data);
                }
                else {
                    return java.lang.Long.valueOf(data);
                }
            }
            else {
                if (hasDecimals) {
                    return java.lang.Float.valueOf(data);
                }
                else {
                    return java.lang.Integer.valueOf(data);
                }
            }
        }
        case 'object': {
            if (!data) {
                return null;
            }
            if (data instanceof Date) {
                return new java.util.Date(data.getTime());
            }
            if (Array.isArray(data)) {
                store = new java.util.ArrayList();
                data.forEach((item) => store.add(serialize(item)));
                return store;
            }
            if (data.native) {
                return data.native;
            }
            store = new java.util.HashMap();
            Object.keys(data).forEach((key) => store.put(key, serialize(data[key])));
            return store;
        }
        default:
            return null;
    }
}
//# sourceMappingURL=index.android.js.map