import { Color, ImageSource } from '@nativescript/core';
import { intoNativeColor } from './common';
export function hueFromColor(color) {
    let nativeColor;
    if (color instanceof Color) {
        nativeColor = color.ios;
    }
    else if (color instanceof UIColor) {
        nativeColor = color;
    }
    if (nativeColor) {
        const hue = new interop.Reference();
        const saturation = new interop.Reference();
        const brightness = new interop.Reference();
        const alpha = new interop.Reference();
        nativeColor.getHueSaturationBrightnessAlpha(hue, saturation, brightness, alpha);
        return hue;
    }
    return null;
}
export function intoNativeMarkerOptions(options) {
    var _a, _b;
    const opts = GMSMarker.new();
    const color = intoNativeColor(options === null || options === void 0 ? void 0 : options.color);
    if (color) {
        opts.icon = GMSMarker.markerImageWithColor(color);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.draggable) === 'boolean') {
        opts.draggable = options.draggable;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.anchorU) === 'number' || typeof (options === null || options === void 0 ? void 0 : options.anchorV) === 'number') {
        const anchorU = (_a = options === null || options === void 0 ? void 0 : options.anchorU) !== null && _a !== void 0 ? _a : opts.groundAnchor.x;
        const anchorV = (_b = options === null || options === void 0 ? void 0 : options.anchorV) !== null && _b !== void 0 ? _b : opts.groundAnchor.y;
        opts.groundAnchor = CGPointMake(anchorU, anchorV);
    }
    if (options === null || options === void 0 ? void 0 : options.position) {
        opts.position = CLLocationCoordinate2DMake(options.position.lat, options.position.lng);
    }
    if (options === null || options === void 0 ? void 0 : options.title) {
        opts.title = options.title;
    }
    if (options === null || options === void 0 ? void 0 : options.snippet) {
        opts.snippet = options.snippet;
    }
    if (options === null || options === void 0 ? void 0 : options.icon) {
        if ((options === null || options === void 0 ? void 0 : options.icon) instanceof UIImage) {
            opts.icon = options.icon;
        }
        else if ((options === null || options === void 0 ? void 0 : options.icon) instanceof ImageSource) {
            opts.icon = options.icon.ios;
        }
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.rotation) === 'number') {
        opts.rotation = options.rotation;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.flat) === 'boolean') {
        opts.flat = options.flat;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex = options.zIndex;
    }
    return opts;
}
export function intoNativeCircleOptions(options) {
    const opts = GMSCircle.new();
    if (typeof (options === null || options === void 0 ? void 0 : options.radius) === 'number') {
        opts.radius = options.radius;
    }
    const strokeColor = intoNativeColor(options === null || options === void 0 ? void 0 : options.strokeColor);
    if (strokeColor !== null) {
        opts.strokeColor = strokeColor;
    }
    const fillColor = intoNativeColor(options === null || options === void 0 ? void 0 : options.fillColor);
    if (fillColor !== null) {
        opts.fillColor = fillColor;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.tappable = options.tappable;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.strokeWidth) === 'number') {
        opts.strokeWidth = options.strokeWidth;
    }
    if (options === null || options === void 0 ? void 0 : options.center) {
        opts.position = CLLocationCoordinate2DMake(options.center.lat, options.center.lng);
    }
    if (Array.isArray(options.strokePattern)) {
        // TODO
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex = options.zIndex;
    }
    return opts;
}
export function intoNativePolygonOptions(options) {
    let path;
    if (Array.isArray(options === null || options === void 0 ? void 0 : options.points)) {
        path = GMSMutablePath.path();
        options.points.forEach((point) => {
            path.addCoordinate(CLLocationCoordinate2DMake(point.lat, point.lng));
        });
    }
    const opts = path ? GMSPolygon.polygonWithPath(path) : GMSPolygon.new();
    if (Array.isArray(options === null || options === void 0 ? void 0 : options.holes)) {
        if (options.holes.length) {
            opts.holes = options.holes.map((hole) => {
                const res = GMSMutablePath.path();
                res.addCoordinate(CLLocationCoordinate2DMake(hole.lat, hole.lng));
            });
        }
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.tappable = options.tappable;
    }
    const strokeColor = intoNativeColor(options === null || options === void 0 ? void 0 : options.strokeColor);
    if (strokeColor !== null) {
        opts.strokeColor = strokeColor;
    }
    const fillColor = intoNativeColor(options === null || options === void 0 ? void 0 : options.fillColor);
    if (fillColor !== null) {
        opts.fillColor = fillColor;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.strokeWidth) === 'number') {
        opts.strokeWidth = options.strokeWidth;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.geodesic) === 'boolean') {
        opts.geodesic = options.geodesic;
    }
    if (options === null || options === void 0 ? void 0 : options.strokeJointType) {
        // TODO;
    }
    if (Array.isArray(options.strokePattern)) {
        // TODO;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex = options.zIndex;
    }
    return opts;
}
export function intoNativePolylineOptions(options) {
    let path;
    if (Array.isArray(options === null || options === void 0 ? void 0 : options.points)) {
        path = GMSMutablePath.path();
        options.points.forEach((point) => {
            path.addCoordinate(CLLocationCoordinate2DMake(point.lat, point.lng));
        });
    }
    const opts = path ? GMSPolyline.polylineWithPath(path) : GMSPolyline.new();
    if (typeof (options === null || options === void 0 ? void 0 : options.width) === 'number') {
        opts.strokeWidth = options.width;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.tappable = options.tappable;
    }
    const color = intoNativeColor(options === null || options === void 0 ? void 0 : options.color);
    if (color !== null) {
        opts.strokeColor = color;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.geodesic) === 'boolean') {
        opts.geodesic = options.geodesic;
    }
    if (options === null || options === void 0 ? void 0 : options.jointType) {
        // TODO
    }
    if (Array.isArray(options.pattern)) {
        // TODO
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex = options.zIndex;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.startCap)) {
        // TODO
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.endCap)) {
        // TODO
    }
    return opts;
}
export function intoNativeGroundOverlayOptions(options) {
    var _a, _b;
    const opts = GMSGroundOverlay.new();
    if (typeof (options === null || options === void 0 ? void 0 : options.width) === 'number') {
        // TODO
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.height) === 'number') {
        // TODO
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.transparency)) {
        // TODO
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.anchorU) === 'number' || typeof (options === null || options === void 0 ? void 0 : options.anchorV) === 'number') {
        opts.anchor = CGPointMake((_a = options === null || options === void 0 ? void 0 : options.anchorU) !== null && _a !== void 0 ? _a : opts.anchor.x, (_b = options === null || options === void 0 ? void 0 : options.anchorV) !== null && _b !== void 0 ? _b : opts.anchor.y);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.tappable = options.tappable;
    }
    if (options === null || options === void 0 ? void 0 : options.position) {
        const coords = options.position;
        opts.position = CLLocationCoordinate2DMake(coords.lat, coords.lng);
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.tappable) === 'boolean') {
        opts.tappable = options.tappable;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.bearing) === 'number') {
        opts.bearing = options.bearing;
    }
    if ((options === null || options === void 0 ? void 0 : options.image) instanceof UIImage) {
        opts.icon = options.image;
    }
    else if ((options === null || options === void 0 ? void 0 : options.image) instanceof ImageSource) {
        opts.icon = options.image.ios;
    }
    if (typeof (options === null || options === void 0 ? void 0 : options.zIndex) === 'number') {
        opts.zIndex = options.zIndex;
    }
    return opts;
}
export function intoNativeTileOverlayOptions(options) { }
export function deserialize(data) {
    if (data instanceof NSNull) {
        return null;
    }
    if (data instanceof NSArray) {
        let array = [];
        for (let i = 0, n = data.count; i < n; i++) {
            array[i] = deserialize(data.objectAtIndex(i));
        }
        return array;
    }
    if (data instanceof NSDictionary) {
        let dict = {};
        for (let i = 0, n = data.allKeys.count; i < n; i++) {
            let key = data.allKeys.objectAtIndex(i);
            dict[key] = deserialize(data.objectForKey(key));
        }
        return dict;
    }
    return data;
}
function numberHasDecimals(item) {
    return !(item % 1 === 0);
}
function numberIs64Bit(item) {
    return item < -Math.pow(2, 31) + 1 || item > Math.pow(2, 31) - 1;
}
export function serialize(data) {
    switch (typeof data) {
        case 'string':
        case 'boolean': {
            return data;
        }
        case 'number': {
            const hasDecimals = numberHasDecimals(data);
            if (numberIs64Bit(data)) {
                if (hasDecimals) {
                    return NSNumber.numberWithDouble(data);
                }
                else {
                    return NSNumber.numberWithLongLong(data);
                }
            }
            else {
                if (hasDecimals) {
                    return NSNumber.numberWithFloat(data);
                }
                else {
                    return data;
                }
            }
        }
        case 'object': {
            if (data instanceof Date) {
                return NSDate.dateWithTimeIntervalSince1970(data.getTime() / 1000);
            }
            if (!data) {
                return null;
            }
            if (Array.isArray(data)) {
                return NSArray.arrayWithArray(data.map(serialize));
            }
            let node = {};
            Object.keys(data).forEach(function (key) {
                let value = data[key];
                node[key] = serialize(value);
            });
            return NSDictionary.dictionaryWithDictionary(node);
        }
        default:
            return null;
    }
}
//# sourceMappingURL=index.ios.js.map